%
% Example 4
% SLIP
%
% Frank Peuker, 2012, Locomorph summer school
%

%
% Additional code units D, E and F (w.r.t. Example 3)
%

clear all

% D Constants
deg=pi/180;

% 1 Integrator parameters
dt=1e-4;
tend=10;
t=0;
i=1;

% 2 Output parameters
dtsample=1e-2;
N=ceil(tend/dtsample);
di=ceil(dtsample/dt);
ts=zeros(N,50);

% 3 Parameters
m=80; 
L0=1;
k=20000;
g=9.81;
alpha=68*deg;

% 4 Initial values
x=0;
y=1;
vx=5;
vy=0;

% A Hybrid system variables
stance_fpx=-100;
stance_fpy=0;
flight_fpx=x+L0*cos(alpha);             
flight_fpy=y-L0*sin(alpha);
fpx=flight_fpx;
fpy=flight_fpy;
yg=0;

% E Variables with content from previous time step
flight_fpy_prev=flight_fpy;

% 5 Just declarations
L=L0;
ax=0;
ay=0;
Fx=0;
Fy=0;
flight=true;                  

% ---------------------------------------------------- Time-looping -------
while t<tend       
    
    % B Event detector
    isTD = flight_fpy < yg && flight_fpy_prev >= yg;
    isLO = L > L0 && not(flight);
    
    % F Event actions
    if isTD
        flight        = false;
        stance_fpx    = flight_fpx;
        stance_fpy    = yg;
    end
    
    if isLO
        flight        = true;
        stance_fpx    = -100;
        stance_fpy    = yg;
    end
    
    % 6 Assemble force 
    
    % 6a, Gravity
    Fx = 0;
    Fy = -m*g; 
    
    % 6b, Repulsive spring force in 2D
    Fxs=0; Fys=0;
    if not(flight)
        L = sqrt( (x-stance_fpx)^2 + (y-stance_fpy)^2 );
        Fval = k*(L0-L);
        Fxs = Fval * (x-stance_fpx)/L; % projection onto x-axis
        Fys = Fval * (y-stance_fpy)/L; % projection onto y-axis
    end
    Fx=Fx+Fxs;
    Fy=Fy+Fys;
    
    % 7 2nd law newton
    ax = Fx/m;
    ay = Fy/m;
    
    % 8 Output
    if mod(i,di)==0        
        % Where is the foot right now? - Depends on state
        if flight
            fpx=flight_fpx;
            fpy=flight_fpy;
        else
            fpx=stance_fpx;
            fpy=stance_fpy;
        end
        
        is=i/di;
        ts(is,1:10)=[t x y fpx fpy vx vy Fxs Fys flight];        
    end
    
    % 9 Euler
    vx = vx + dt * ax;
    vy = vy + dt * ay;
    x = x + dt * vx;
    y = y + dt * vy;
    
    % C Update variables
    flight_fpy_prev=flight_fpy;
    flight_fpx=x+L0*cos(alpha);
    flight_fpy=y-L0*sin(alpha);
       
    % 10 Advance in time
    t=t+dt;
    i=i+1;
end
