%
% Example 5
% SLIP  
%   a) with step counter 
%   b) with quit conditions: i.e. steps <= 50 AND not falling (i.e. y > 0)
%   c) implemented as function
%   d) flag to suppress output (dramatic speed-up)
%
%   Function is suitable to record STEPS TO FALL MAP
%
% Frank Peuker, 2012, Locomorph summer school
%

function steps=slipFcn(k,alpha)

% D Constants
deg=pi/180;

% 1 Integrator parameters
dt=1e-4;
tend=40;
t=0;
i=1;
quit=false;
isOutput=false;

% 2 Output parameters
if isOutput
    dtsample=1e-2;
    N=ceil(tend/dtsample);
    di=ceil(dtsample/dt);
    ts=zeros(N,50);
end

% 3 Parameters
m=80; 
L0=1;
%k=20000;
g=9.81;
%alpha=68*deg;

% 4 Initial values
x=0;
y=1;
vx=5;
vy=0;

% A Hybrid system variables
stance_fpx=-100;
stance_fpy=0;
flight_fpx=x+L0*cos(alpha);             
flight_fpy=y-L0*sin(alpha);
fpx=flight_fpx;
fpy=flight_fpy;
yg=0;

% E Variables with content from previous time step
flight_fpy_prev=flight_fpy;

% 5 Just declarations
L=L0;
ax=0;
ay=0;
Fx=0;
Fy=0;
flight=true;
steps=0;

% ---------------------------------------------------- Time-looping -------
while t<tend && not(quit)    
    
    % B Event detector
    isTD = flight_fpy < yg && flight_fpy_prev >= yg;
    isLO = L > L0 && not(flight);
    
    % F Event actions
    if isTD
        flight        = false;
        stance_fpx    = flight_fpx;
        stance_fpy    = yg;
        steps         = steps+1;
    end
    
    if isLO
        flight        = true;
        stance_fpx    = -100;
        stance_fpy    = yg;
    end
    
    % 6 Assemble force 
    
    % 6a, Gravity
    Fx = 0;
    Fy = -m*g; 
    
    % 6b, Repulsive spring force in 2D
    Fxs=0; Fys=0;
    if not(flight)
        L = sqrt( (x-stance_fpx)^2 + (y-stance_fpy)^2 );
        Fval = k*(L0-L);
        Fxs = Fval * (x-stance_fpx)/L; % projection onto x-axis
        Fys = Fval * (y-stance_fpy)/L; % projection onto y-axis
    end
    Fx=Fx+Fxs;
    Fy=Fy+Fys;
    
    % 7 2nd law newton
    ax = Fx/m;
    ay = Fy/m;
    
    % 8 Output
    if isOutput
        if mod(i,di)==0
            % Where is the foot right now? - Depends on state
            if flight
                fpx=flight_fpx;
                fpy=flight_fpy;
            else
                fpx=stance_fpx;
                fpy=stance_fpy;
            end
            
            is=i/di;
            ts(is,1:10)=[t x y fpx fpy vx vy Fxs Fys flight];
        end
    end
    
    % 9 Euler
    vx = vx + dt * ax;
    vy = vy + dt * ay;
    x = x + dt * vx;
    y = y + dt * vy;
    
    % C Update variables
    flight_fpy_prev=flight_fpy;
    flight_fpx=x+L0*cos(alpha);
    flight_fpy=y-L0*sin(alpha);
    quit=steps>50 || y<0;
       
    % 10 Advance in time
    t=t+dt;
    i=i+1;
end

end